﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using FirstRunTutorial.wp.Resources;
using System.Windows.Media.Animation;
using System.IO.IsolatedStorage;

namespace FirstRunTutorial.wp
{
    public partial class MainPage : PhoneApplicationPage
    {
        private bool _isNewPageInstance = false;
        private bool _firstRunActive = false;
        private int _firstRunIndex = 0;
        // TODO - this should match the number of tutorial screens
        private const int MAX_FIRSTRUNINDEX = 4;

        // Constructor
        public MainPage()
        {
            InitializeComponent();
            _isNewPageInstance = true;

            CheckFirstRun();

            // Sample code to localize the ApplicationBar
            //BuildLocalizedApplicationBar();
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            if (_isNewPageInstance)
            {
                if (State.Count > 0)
                {
                    SetFirstRunActive((bool)State["FirstRunActive"]);
                    _firstRunIndex = (int)State["FirstRunIndex"];
                    UpdateFirstRunCanvas();
                }
            }
            if (e.NavigationMode != System.Windows.Navigation.NavigationMode.Back)
            {
                if (NavigationContext.QueryString.ContainsKey("RemoveBackAll"))
                {
                    var bs = NavigationService.BackStack;
                    int numToRemove = 0;
                    foreach (JournalEntry je in bs)
                    {
                        numToRemove++;
                        if (je.Source.OriginalString.StartsWith("/MainPage.xaml"))
                        {
                            break;
                        }
                    }
                    for (int i = 0; i < numToRemove; i++)
                    {
                        if (NavigationService.CanGoBack)
                        {
                            NavigationService.RemoveBackEntry();
                        }
                    }

                    // So we don't remove them again if we go to a further scene then back here
                    NavigationContext.QueryString.Remove("RemoveBackAll");
                }
                if (NavigationContext.QueryString.ContainsKey("Tutorial"))
                {
                    _firstRunIndex = 0;
                    SetFirstRunActive(true);
                    NavigationContext.QueryString.Remove("Tutorial");
                    UpdateFirstRunCanvas();
                }
            }
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            base.OnNavigatedFrom(e);
            if (e.NavigationMode != NavigationMode.Back)
            {
                State["FirstRunActive"] = _firstRunActive;
                State["FirstRunIndex"] = _firstRunIndex;
            }
            else
            {
                State.Clear();
            }
        }

        private DateTime? _lastFirstRunTap = null;
        private void FirstRunCanvas_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            if (_firstRunActive)
            {
                // don't allow taps too fast to prevent accidents
                if (_lastFirstRunTap.HasValue &&
                    DateTime.Now.Subtract(_lastFirstRunTap.Value).CompareTo(TimeSpan.FromMilliseconds(500)) < 0)
                {
                    return;
                }
                _lastFirstRunTap = DateTime.Now;
                _firstRunIndex++;
                UpdateFirstRunCanvas();
            }
        }

        private void CheckFirstRun()
        {
            if (FirstRunSettings.FakeFirstRun ||
                (!IsolatedStorageSettings.ApplicationSettings.Contains("LastRun")))
            {
                SetFirstRunActive(true);
                _firstRunIndex = 0;
            }
            else
            {
                SetFirstRunActive(false);
            }
            IsolatedStorageSettings.ApplicationSettings["LastRun"] = DateTime.Now;
            UpdateFirstRunCanvas();
        }

        private void SetFirstRunActive(bool firstRunActive)
        {
            _firstRunActive = firstRunActive;
            // TODO - add more controls here that shouldn't be able to be interacted
            // with while the tutorial is active
            SampleButton.IsHitTestVisible = !firstRunActive;
        }


        private void UpdateFirstRunCanvas()
        {
            // check if done already
            if (_firstRunIndex >= MAX_FIRSTRUNINDEX)
            {
                SetFirstRunActive(false);
                _firstRunIndex = 0;
            }
            FirstRunCanvas.Visibility = _firstRunActive ? Visibility.Visible : Visibility.Collapsed;
            double mainOpacity = _firstRunActive ? 0.6 : 1.0;
            // TODO - add more controls to dim out here
            SampleText.Opacity = mainOpacity;
            SampleButton.Opacity = mainOpacity;

            if (_firstRunActive)
            {
                if (_firstRunIndex == 0)
                {
                    FirstRunFirstShowStoryboard.Begin();
                    FirstRunFirst.Visibility = System.Windows.Visibility.Visible;
                }
                else
                {
                    FirstRunFirstHideStoryboard.Begin();
                }
                if (_firstRunIndex == 1)
                {
                    FirstRunSecondShowStoryboard.Begin();
                    FirstRunSecond.Visibility = System.Windows.Visibility.Visible;
                }
                else
                {
                    FirstRunSecondHideStoryboard.Begin();
                }
                if (_firstRunIndex == 2)
                {
                    FirstRunButtonShowStoryboard.Begin();
                    FirstRunButton.Visibility = System.Windows.Visibility.Visible;
                    SampleButtonHighlightStoryboard.Begin();
                }
                else
                {
                    FirstRunButtonHideStoryboard.Begin();
                    SampleButtonHighlightStoryboard.Stop();
                }
                if (_firstRunIndex == 3)
                {
                    FirstRunFinalShowStoryboard.Begin();
                    FirstRunFinal.Visibility = System.Windows.Visibility.Visible;
                }
                else
                {
                    FirstRunFinalHideStoryboard.Begin();
                }
                // TODO - add more steps here
            }
            else
            {
                FirstRunFirstHideStoryboard.Begin();
                FirstRunSecondHideStoryboard.Begin();
                FirstRunButtonHideStoryboard.Begin();
                SampleButtonHighlightStoryboard.Stop();
                FirstRunFinalHideStoryboard.Begin();
                // TODO - add more steps here
            }
        }

        protected override void OnBackKeyPress(System.ComponentModel.CancelEventArgs e)
        {
            if (_firstRunActive)
            {
                if (_firstRunIndex > 0)
                {
                    _firstRunIndex--;
                }
                else
                {
                    SetFirstRunActive(false);
                }
                UpdateFirstRunCanvas();
                e.Cancel = true;
                return;
            }
            base.OnBackKeyPress(e);
        }

        private void HideWhenDone(object sender, EventArgs e)
        {
            string targetName = (string)((DependencyObject)(sender)).GetValue(Storyboard.TargetNameProperty);
            if (targetName != null)
            {
                FrameworkElement elem = this.FindName(targetName) as FrameworkElement;
                if (elem != null)
                {
                    elem.Visibility = System.Windows.Visibility.Collapsed;
                }
            }
        }

        private void About_Click(object sender, EventArgs e)
        {
            NavigationService.Navigate(new Uri("/About.xaml", UriKind.Relative));
        }

        // Sample code for building a localized ApplicationBar
        //private void BuildLocalizedApplicationBar()
        //{
        //    // Set the page's ApplicationBar to a new instance of ApplicationBar.
        //    ApplicationBar = new ApplicationBar();

        //    // Create a new button and set the text value to the localized string from AppResources.
        //    ApplicationBarIconButton appBarButton = new ApplicationBarIconButton(new Uri("/Assets/AppBar/appbar.add.rest.png", UriKind.Relative));
        //    appBarButton.Text = AppResources.AppBarButtonText;
        //    ApplicationBar.Buttons.Add(appBarButton);

        //    // Create a new menu item with the localized string from AppResources.
        //    ApplicationBarMenuItem appBarMenuItem = new ApplicationBarMenuItem(AppResources.AppBarMenuItemText);
        //    ApplicationBar.MenuItems.Add(appBarMenuItem);
        //}
    }
}